<?php
/**
 * @file
 * Define pages for the Deputy module.
 */

/**
 * Form callback to toggle on and off Deputy Systems. 
 */
function deputy_settings_form($form_state) {
  deputy_include_enabled();

  $form = array(
    'description' => array(
      '#value' => t('When saving new settings there will be a delay for cache clearing.') . '<br/><br/>',
    ),
  );

  $form['deputy_systems'] = array(
    '#tree' => TRUE,
  );

  foreach (deputy_system_load() as $name => $system) {
    if (empty($system['final'])) {
      $form['deputy_systems'][$name] = array(
        '#title' => $system['title'],
        '#description' => $system['description'],
        '#type' => 'checkbox',
        '#access' => deputy_admin_access($name),
        '#default_value' => deputy_system_exists($name),
      );

      if ($form['deputy_systems'][$name]['#default_value'] && ($link = _deputy_configuration_link($system))) {
        $form['deputy_systems'][$name]['#description'] .= ' ' . $link;
      }
    }
  }

  // Handle deputy systems, then remove before system_settings_form_submit can touch it.
  $form['#submit'][] = 'deputy_system_status_submit';
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  if (!empty($_POST) && form_get_errors()) {
    drupal_set_message(t('The settings have not been saved because of the errors.'), 'error');
  }

  return $form;
}

/**
 * Construct a form for a deputy system settings callback.
 */
function deputy_system_form(&$form_state, $system) {
  deputy_include($system['name']);
  $form = $system['settings callback']($form_state, $system);
  $form['#system'] = $system;

  if (empty($_POST) && $link = _deputy_configuration_link($system, t('extended configuration options'))) {
    drupal_set_message(t('There are !extended for this system.', array(
      '!extended' => $link,
    )), FALSE);
  }

  return empty($form['#submit']) ? system_settings_form($form) : $form;
}

/**
 * Submit callback to handle activation/deactivation of Deputy systems.
 */
function deputy_system_status_submit($form, &$form_state) {
  if (!empty($form_state['values']['deputy_systems'])) {
    foreach ($form_state['values']['deputy_systems'] as $system => $enable) {
      if (deputy_system_exists($system) != $enable) {
        $cc = TRUE;
        if ($enable) {
          deputy_system_enable($system);
        }
        else {
          deputy_system_disable($system);
        }
      }
    }
  }

  if ($cc) {
    variable_set('perm_conf_update_needed', 1);
    drupal_flush_all_caches();
  }
}

/**
 * If the current user has permission, generate a link to further configuration.
 */
function _deputy_configuration_link($system, $text = FALSE) {
  if (!empty($system['configure'])) {
    $menu = menu_get_item($system['configure']);
    if ($menu['access']) {
      $text = $text ? $text : t('Configure');
      return l($text, $system['configure']);
    }
  }

  return '';
}
